@@ -2,7 +2,7 @@
  *
  * display.c - displaying of timing and messages
  *
- * Copyright © 2006 Scott James Remnant <scott@netsplit.com>.
+ * Copyright © 2010 Scott James Remnant <scott@netsplit.com>.
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 #include <string.h>
 #include <curses.h>
 #include <time.h>
+#include <regex.h>
 
 #include "live-f1.h"
 #include "packet.h" /* for packet type */
@@ -133,9 +134,9 @@ open_display (void)
 		attrs[COLOUR_OLD]         = COLOR_PAIR (COLOUR_OLD);
 		attrs[COLOUR_ELIMINATED]  = COLOR_PAIR (COLOUR_ELIMINATED) | A_BOLD;
 		attrs[COLOUR_POPUP]       = COLOR_PAIR (COLOUR_POPUP) | A_BOLD;
-		attrs[COLOUR_GREEN_FLAG]  = COLOR_PAIR (COLOUR_GREEN_FLAG);
-		attrs[COLOUR_YELLOW_FLAG] = COLOR_PAIR (COLOUR_YELLOW_FLAG);
-		attrs[COLOUR_RED_FLAG]    = COLOR_PAIR (COLOUR_RED_FLAG);
+		attrs[COLOUR_GREEN_FLAG]  = COLOR_PAIR (COLOUR_GREEN_FLAG) | A_REVERSE;
+		attrs[COLOUR_YELLOW_FLAG] = COLOR_PAIR (COLOUR_YELLOW_FLAG) | A_REVERSE;
+		attrs[COLOUR_RED_FLAG]    = COLOR_PAIR (COLOUR_RED_FLAG) | A_REVERSE;
 	}
 
 	bkgdset (attrs[COLOUR_DEFAULT]);
@@ -163,10 +164,10 @@ clear_board (CurrentState *state)
 	if (boardwin)
 		delwin (boardwin);
 
-	nlines = MAX (state->num_cars, 22);
+	nlines = MAX (state->num_cars, 21);
 	for (i = 0; i < state->num_cars; i++)
 		nlines = MAX (nlines, state->car_position[i]);
-	nlines += 1;
+	nlines += 3;
 	/*if (LINES < nlines) {
 		close_display ();
 		fprintf (stderr, "%s: %s\n", program_name,
@@ -181,31 +182,31 @@ clear_board (CurrentState *state)
 	}*/
 
 	boardwin = newwin (nlines, 69, 0, 0);
-	wbkgdset (boardwin, attrs[COLOUR_DEFAULT]);
+	wbkgdset (boardwin, attrs[COLOUR_DATA]);
 	werase (boardwin);
 
-	switch (state->event_type) {
-	case RACE_EVENT:
-		mvwprintw (boardwin, 0, 0,
-			   "%2s %2s %-14s %4s %4s %-8s %-8s %-8s %-8s %2s",
-			   _("P"), _(""), _("Name"), _("Gap"), _("Int"),
-			   _("Time"), _("Sector 1"), _("Sector 2"),
-			   _("Sector 3"), _("Ps"));
-		break;
-	case PRACTICE_EVENT:
-		mvwprintw (boardwin, 0, 0,
-			   "%2s %2s %-14s %-8s %6s %5s %5s %5s %-4s",
-			   _("P"), _(""), _("Name"), _("Best"), _("Gap"),
-			   _("Sec 1"), _("Sec 2"), _("Sec 3"), _("Laps"));
-		break;
-	case QUALIFYING_EVENT:
-		mvwprintw (boardwin, 0, 0,
-			   "%2s %2s %-14s %-8s %-8s %-8s %5s %5s %5s %-2s",
-			   _("P"), _(""), _("Name"), _("Period 1"),
-			   _("Period 2"), _("Period 3"), _("Sec 1"),
-			   _("Sec 2"), _("Sec 3"), _("Ls"));
-		break;
-	}
+		switch (state->event_type) {
+		case RACE_EVENT:
+			mvwprintw (boardwin, 0, 0,
+				   "%2s %2s %-14s %4s %4s %-8s %-8s %-8s %-8s %2s",
+				   _("P"), _(""), _("Name"), _("Gap"), _("Int"),
+				   _("Time"), _("Sector 1"), _("Sector 2"),
+				   _("Sector 3"), _("Ps"));
+			break;
+		case PRACTICE_EVENT:
+			mvwprintw (boardwin, 0, 0,
+				   "%2s %2s %-14s %-8s %6s %5s %5s %5s %-4s",
+				   _("P"), _(""), _("Name"), _("Best"), _("Gap"),
+				   _("Sec 1"), _("Sec 2"), _("Sec 3"), _(" Lap"));
+			break;
+		case QUALIFYING_EVENT:
+			mvwprintw (boardwin, 0, 0,
+				   "%2s %2s %-14s %-8s %-8s %-8s %5s %5s %5s %-2s",
+				   _("P"), _(""), _("Name"), _("Period 1"),
+				   _("Period 2"), _("Period 3"), _("Sec 1"),
+				   _("Sec 2"), _("Sec 3"), _("Lp"));
+			break;
+		}
 
 	for (i = 1; i <= state->num_cars; i++) {
 		for (j = 0; j < LAST_CAR_PACKET; j++)
@@ -363,7 +364,7 @@ _update_cell (CurrentState *state,
 			sz = 5;
 			align = 1;
 			break;
-		case PRACTICE_LAPS:
+		case PRACTICE_LAP:
 			x = 55;
 			sz = 4;
 			align = 1;
@@ -419,7 +420,7 @@ _update_cell (CurrentState *state,
 			sz = 5;
 			align = 1;
 			break;
-		case QUALIFYING_LAPS:
+		case QUALIFYING_LAP:
 			x = 66;
 			sz = 2;
 			align = 1;
@@ -541,6 +542,25 @@ clear_car (CurrentState *state,
 	doupdate ();
 }
 
+/**
+ * draw_bar:
+ * @win: the window into which to draw
+ * @len: the length of the bar to draw
+ *
+ * Draws a solid bar of a specified length, at the current cursor
+ * position in the specified window.
+ **/
+void
+draw_bar (WINDOW *win, int len)
+{
+	int i;
+
+	for (i = 0; i < len; i++)
+	{
+		waddch (win, ' ');
+	}
+}
+
 /**
  * update_status:
  * @state: application state structure,
@@ -561,43 +581,53 @@ update_status (CurrentState *state)
 			return;
 
 		statwin = newwin (nlines, 10, 0, COLS - 10);
-		wbkgdset (statwin, attrs[COLOUR_DEFAULT]);
+		wbkgdset (statwin, attrs[COLOUR_DATA]);
 		werase (statwin);
 	}
 
-	/* Output the race status */
-	wmove (statwin, 1, 0);
+	/* Session status */
+
+	wmove (statwin, 2, 0);
 	wclrtoeol (statwin);
 	switch (state->flag) {
-	case GREEN_FLAG:
-		wattrset (statwin, attrs[COLOUR_GREEN_FLAG]);
-		break;
 	case YELLOW_FLAG:
-		wattrset (statwin, attrs[COLOUR_YELLOW_FLAG]);
-		break;
 	case SAFETY_CAR_STANDBY:
-		wattrset (statwin, attrs[COLOUR_YELLOW_FLAG]);
-		waddstr (statwin, "SC Standby");
-		break;
 	case SAFETY_CAR_DEPLOYED:
 		wattrset (statwin, attrs[COLOUR_YELLOW_FLAG]);
-		waddstr (statwin, "Safety Car");
+		draw_bar (statwin, 10);
 		break;
 	case RED_FLAG:
 		wattrset (statwin, attrs[COLOUR_RED_FLAG]);
-		waddstr (statwin, "Stopped");
+		draw_bar (statwin, 10);
 		break;
-	default:
-		wattrset (statwin, attrs[COLOUR_DEFAULT]);
+	}
+	wmove (statwin, 3, 0);
+	wclrtoeol (statwin);
+	switch (state->flag) {
+	case SAFETY_CAR_DEPLOYED:
+		wattrset (statwin, attrs[COLOUR_OLD]);
+		waddstr (statwin, "SAFETY CAR");
 		break;
 	}
 
-	/* Number of laps, or event type (use same colour as flag) */
+	/* Number of laps, or event type */
+
+	wattrset (statwin, attrs[COLOUR_DATA]);
 	wmove (statwin, 0, 0);
 	wclrtoeol (statwin);
 	switch (state->event_type) {
 	case RACE_EVENT:
-		wprintw (statwin, "Lap %3d", state->lap);
+		switch (state->total_laps - state->laps_completed) {
+		case 0:
+			wprintw (statwin, "%10s", "FINISHED");
+			break;
+		case 1:
+			wprintw (statwin, "%10s", "FINAL LAP");
+			break;
+		default:
+			wprintw (statwin, "%4d TO GO", state->total_laps - state->laps_completed);
+			break;
+		}
 		break;
 	case PRACTICE_EVENT:
 		wprintw (statwin, "Practice");
@@ -607,9 +637,78 @@ update_status (CurrentState *state)
 		break;
 	}
 
+	/* Display weather */
+
+	int wline = 5;
+	wattrset (statwin, attrs[COLOUR_DATA]);
+ 
+	wmove (statwin, wline, 0);
+	wclrtoeol (statwin);
+	wprintw(statwin,"%-6s%2d C", "Track", state->track_temp);
+	wmove (statwin, wline, 8);
+	waddch (statwin, ACS_DEGREE);
+
+	wline += 2;
+
+	wmove (statwin, wline, 0);
+	wclrtoeol (statwin);
+	wprintw(statwin,"%-6s%2d C", "Air", state->air_temp);
+	wmove (statwin, wline, 8);
+	waddch (statwin, ACS_DEGREE);
+
+	wline += 2;
+
+	wmove (statwin, wline, 0);
+	wclrtoeol (statwin);
+	wprintw (statwin, "%-6s%3d", "Wind", state->wind_direction);
+	waddch (statwin, ACS_DEGREE);
+	wline++;
+	wmove (statwin, wline, 0);
+	wclrtoeol (statwin);
+	wprintw (statwin, "%-4s%03dm/s", "", state->wind_speed);
+	wmove (statwin, wline, 5);
+	waddch (statwin, '.');
+
+	wline += 2;
+
+	wmove (statwin, wline, 0);
+	wclrtoeol (statwin);
+	wprintw(statwin, "Humidity");
+	wline++;
+	wmove (statwin, wline, 0);
+	wclrtoeol (statwin);
+	wprintw (statwin, "%-6s%3d%%", "", state->humidity);
+
+	wline += 2;
+
+	wmove (statwin, wline, 0);
+	wclrtoeol (statwin);
+	wprintw(statwin, "Pressure");
+	wline++;
+	wmove (statwin, wline, 0);
+	wclrtoeol (statwin);
+	wprintw(statwin, "%-2s%6dmb", "", state->pressure);
+	wmove (statwin, wline, 6);
+	waddch (statwin, '.');
+
+	/* Update fastest lap line (race only) */
+
+	if (state->event_type == RACE_EVENT)
+	{
+		wmove (boardwin, nlines - 1, 3);
+		wattrset (boardwin, attrs[COLOUR_RECORD]);
+		wclrtoeol (boardwin);
+		wprintw(boardwin, "%2s %-14s %4s %4s %8s", state->fl_car, state->fl_driver, "LAP", state->fl_lap, state->fl_time);
+	}
+
+	/* Update session clock */
+	
 	_update_time (state);
 
+	/* Refresh display */
+
 	wnoutrefresh (statwin);
+	wnoutrefresh (boardwin);
 	doupdate ();
 }
 
@@ -627,12 +726,15 @@ _update_time (CurrentState *state)
 	if (! statwin)
 		return;
 
-	wmove (statwin, nlines - 1, 0);
+	wmove (statwin, nlines - 1, 2);
 	wattrset (statwin, attrs[COLOUR_DATA]);
 
-	if (state->epoch_time && (state->flag != RED_FLAG)) {
-		remaining = MAX ((state->epoch_time + state->remaining_time)
-				 - time (NULL), 0);
+	// Pause the clock during a red flag, but only for Qualifying and the Race.
+	if ((state->flag == RED_FLAG) && (state->event_type != PRACTICE_EVENT))
+	{
+		remaining = state->remaining_time;
+	} else if (state->epoch_time) {
+		remaining = MAX ((state->epoch_time + state->remaining_time) - time (NULL), 0);
 	} else {
 		remaining = state->remaining_time;
 	}
@@ -644,11 +746,11 @@ _update_time (CurrentState *state)
 		remaining %= 60;
 		wprintw (statwin, "%02d", remaining);
 	} else if (remaining >= 60) {
-		wprintw (statwin, "  %2d:", remaining / 60);
+		wprintw (statwin, "0:%02d:", remaining / 60);
 		remaining %= 60;
 		wprintw (statwin, "%02d", remaining);
 	} else {
-		wprintw (statwin, "     %2d", remaining);
+		wprintw (statwin, "0:00:%02d", remaining);
 	}
 
 	wnoutrefresh (statwin);
@@ -740,11 +842,21 @@ popup_message (const char *message)
 	char  *msg;
 	size_t msglen;
 	int    nlines, ncols, col, ls, i;
+	regex_t re;
 
 	open_display ();
 	close_popup ();
 
-	msg = strdup (message);
+	regcomp(&re, "^img:", REG_EXTENDED|REG_NOSUB);
+
+	if (regexec(&re, message, (size_t)0, NULL, 0) == 0)
+	{
+		msg = strdup ("CURRENTLY NO LIVE SESSION");
+	} else {
+		msg = strdup (message);
+	}
+	regfree (&re);
+
 	msglen = strlen (msg);
 	while (msglen && strchr(" \t\r\n", msg[msglen - 1]))
 		msg[--msglen] = 0;
